from pathlib import Path
from typing import Optional

from pydantic import field_serializer
from pydantic_core.core_schema import FieldSerializationInfo
from pydantic_settings import BaseSettings, SettingsConfigDict

from qualibrate_app.config.models.storage import (
    StorageSettings,
    StorageSettingsBase,
    StorageSettingsSetup,
)
from qualibrate_app.config.models.versioned import Versioned

__all__ = [
    "QualibrateSettings",
    "QualibrateSettingsBase",
    "QualibrateSettingsSetup",
]


class QualibrateSettingsBase(BaseSettings, Versioned):
    model_config = SettingsConfigDict(extra="ignore")

    project: Optional[str]
    storage: StorageSettingsBase
    log_folder: Optional[Path] = None


class QualibrateSettingsSetup(QualibrateSettingsBase):
    storage: StorageSettingsSetup

    @field_serializer("project")
    def serialize_project(
        self, value: Optional[str], _info: FieldSerializationInfo
    ) -> str:
        return value or ""


class QualibrateSettings(QualibrateSettingsBase):
    project: str
    storage: StorageSettings



from pathlib import Path
from typing import Optional

from pydantic import BaseModel, field_serializer

from qualibrate_app.config.models.path_serializer import PathSerializer

__all__ = [
    "ActiveMachineSettings",
    "ActiveMachineSettingsBase",
    "ActiveMachineSettingsSetup",
]


class ActiveMachineSettingsBase(BaseModel, PathSerializer):
    path: Optional[Path] = None


class ActiveMachineSettingsSetup(ActiveMachineSettingsBase):
    path_serializer = field_serializer("path")(PathSerializer.serialize_path)


class ActiveMachineSettings(ActiveMachineSettingsBase):
    # path: DirectoryPath # TODO: require directory
    pass



from pathlib import Path
from typing import ClassVar

from pydantic import DirectoryPath, field_serializer
from pydantic_settings import BaseSettings, SettingsConfigDict

from qualibrate_app.config.models.active_machine import (
    ActiveMachineSettings,
)
from qualibrate_app.config.models.path_serializer import PathSerializer
from qualibrate_app.config.models.qualibrate import QualibrateSettings
from qualibrate_app.config.models.remote_services import (
    JsonTimelineDBBase,
    QualibrateRunnerBase,
)
from qualibrate_app.config.models.versioned import Versioned

__all__ = [
    "QualibrateAppSettings",
    "QualibrateAppSettingsBase",
    "QualibrateAppSettingsSetup",
]


class QualibrateAppSettingsBase(BaseSettings, PathSerializer, Versioned):
    model_config: ClassVar[SettingsConfigDict] = SettingsConfigDict(
        extra="ignore",
    )
    static_site_files: Path
    metadata_out_path: str

    timeline_db: JsonTimelineDBBase
    runner: QualibrateRunnerBase


class QualibrateAppSettingsSetup(QualibrateAppSettingsBase):
    static_files_serializer = field_serializer("static_site_files")(
        PathSerializer.serialize_path
    )


class QualibrateAppSettings(QualibrateAppSettingsBase):
    static_site_files: DirectoryPath
    qualibrate: QualibrateSettings
    active_machine: ActiveMachineSettings



import warnings
from collections.abc import Mapping
from copy import deepcopy
from pathlib import Path
from typing import (
    Any,
    Optional,
    TypeVar,
)

import click
from pydantic import ValidationError

from qualibrate_app.config import read_config_file
from qualibrate_app.config.models.active_machine import (
    ActiveMachineSettingsBase,
)
from qualibrate_app.config.models.qualibrate import (
    QualibrateSettingsBase,
)
from qualibrate_app.config.models.qualibrate_app import (
    QualibrateAppSettingsBase,
)
from qualibrate_app.config.vars import (
    ACTIVE_MACHINE_CONFIG_KEY,
    CONFIG_KEY,
    QUALIBRATE_CONFIG_KEY,
)

T = TypeVar(
    "T",
    QualibrateAppSettingsBase,
    QualibrateSettingsBase,
    ActiveMachineSettingsBase,
)

SUGGEST_MSG = (
    "Can't parse existing config. Fix it or overwrite "
    "by default values using `--overwrite` flag."
)


def get_config_solved_references_or_print_error(
    config_path: Path,
) -> Optional[dict[str, Any]]:
    try:
        return read_config_file(config_path, solve_references=True)
    except ValueError as ex:
        click.secho(str(ex), fg="red")
        click.secho(SUGGEST_MSG, fg="yellow")
    return None


def check_config_pre_v1_and_update(
    common_config: dict[str, Any], qapp_config: dict[str, Any]
) -> tuple[dict[str, Any], dict[str, Any], dict[str, Any]]:
    if "config_version" in qapp_config:
        return (
            common_config[QUALIBRATE_CONFIG_KEY],
            common_config[ACTIVE_MACHINE_CONFIG_KEY],
            qapp_config,
        )
    warnings.warn(
        UserWarning(
            "You are using old version of config. "
            "Please update to new structure."
        ),
        stacklevel=2,
    )
    qapp_config = dict(deepcopy(qapp_config))
    qualibrate_config = {
        "storage": {
            "type": qapp_config.pop("storage_type"),
            "location": qapp_config.pop("user_storage").replace(
                f"#/{CONFIG_KEY}/project",
                f"#/{QUALIBRATE_CONFIG_KEY}/project",
            ),
        },
        "project": qapp_config.pop("project"),
    }
    active_machine_config = {}
    if "active_machine_path" in qapp_config:
        active_machine_config["path"] = qapp_config.pop("active_machine_path")
    return qualibrate_config, active_machine_config, qapp_config


def get_config_model_or_print_error(
    config: Mapping[str, Any],
    model_type: type[T],
    config_key: str,
) -> Optional[T]:
    try:
        return model_type(**config)
    except ValidationError as ex:
        errors = [
            (
                f"Message: {error.get('msg')}. "
                "Path: "
                f"{'.'.join([config_key, *map(str, error.get('loc', []))])}. "  # type: ignore
                f"Value: {error.get('input')}"
            )
            for error in ex.errors()
        ]
        click.secho("\n".join(errors), fg="red")
        click.secho(SUGGEST_MSG, fg="yellow")
    return None



from pathlib import Path

from pydantic import DirectoryPath, field_serializer
from pydantic_core.core_schema import FieldSerializationInfo
from pydantic_settings import BaseSettings

from qualibrate_app.config.models.path_serializer import PathSerializer
from qualibrate_app.config.models.storage_type import StorageType

__all__ = ["StorageSettings", "StorageSettingsBase", "StorageSettingsSetup"]


class StorageSettingsBase(BaseSettings, PathSerializer):
    type: StorageType = StorageType.local_storage
    location: Path


class StorageSettingsSetup(StorageSettingsBase):
    @field_serializer("type")
    def serialize_storage_type(
        self, value: StorageType, _info: FieldSerializationInfo
    ) -> str:
        return value.value

    location_serializer = field_serializer("location")(
        PathSerializer.serialize_path
    )


class StorageSettings(StorageSettingsBase):
    location: DirectoryPath



from collections.abc import Sequence
from datetime import datetime
from typing import Union, cast
from urllib.parse import urljoin

import requests
from pydantic import ValidationError

from qualibrate_app.api.core.domain.bases.project import ProjectsManagerBase
from qualibrate_app.api.core.models.project import Project
from qualibrate_app.api.core.utils.request_utils import request_with_db
from qualibrate_app.api.exceptions.classes.timeline_db import QJsonDbException
from qualibrate_app.api.exceptions.classes.values import QValueException
from qualibrate_app.config import (
    QUALIBRATE_CONFIG_KEY,
    QualibrateSettings,
    QualibrateSettingsSetup,
)


class ProjectsManagerTimelineDb(ProjectsManagerBase):
    def _check_db_project_exists(self, project_name: str) -> bool:
        response = request_with_db(
            "database/connect",
            db_name=project_name,
            host=self._settings.timeline_db.address_with_root,
            timeout=self._settings.timeline_db.timeout,
        )
        return response.status_code == 200 and response.json() is True

    def _active_project_setter(self, value: str) -> None:
        if not self._check_db_project_exists(value):
            raise QJsonDbException(
                f"Can't check if project {value} exists in timeline DB."
            )
        self._set_user_storage_project(value)

    def _set_user_storage_project(self, project_name: str) -> None:
        raw_config, new_config = self._get_raw_and_resolved_ref_config(
            project_name
        )
        qs_dict = new_config.get(QUALIBRATE_CONFIG_KEY, {})
        qs: Union[QualibrateSettings, QualibrateSettingsSetup]
        try:
            qs = QualibrateSettings(**qs_dict)
        except ValidationError as ex:
            errors = ex.errors(include_url=False, include_input=False)
            if len(errors) != 1 or not (
                errors[0]["type"] == "path_not_directory"
            ):
                raise
            qs = QualibrateSettingsSetup(**qs_dict)
        self._settings.qualibrate.project = cast(str, qs.project)
        self._settings.qualibrate.storage.location = qs.storage.location

    def create(self, project_name: str) -> str:
        if any(project.name == project_name for project in self.list()):
            raise QValueException(f"Project {project_name} already exists.")
        response = request_with_db(
            "database/create",
            db_name=project_name,
            host=self._settings.timeline_db.address_with_root,
            timeout=self._settings.timeline_db.timeout,
            method=requests.post,
        )

        if response.status_code != 200 or response.json() != project_name:
            raise QJsonDbException(f"Can't create project {project_name}.")
        new_project_path = self._resolve_new_project_path(
            project_name,
            self._settings.qualibrate.project,
            self._settings.qualibrate.storage.location,
        )
        new_project_path.mkdir(parents=True, exist_ok=True)
        return project_name

    def list(self) -> Sequence[Project]:
        response = requests.get(
            urljoin(
                self._settings.timeline_db.address_with_root, "database/list"
            ),
            timeout=self._settings.timeline_db.timeout,
        )
        if response.status_code != 200:
            raise QJsonDbException("Can't resolve list of projects.")
        return list(
            Project(
                name=name,
                nodes_number=-1,
                created_at=datetime.fromtimestamp(0),
                last_modified_at=datetime.fromtimestamp(0),
            )
            for name in response.json()
        )



from .active_machine import ActiveMachineSettings, ActiveMachineSettingsSetup
from .path_serializer import PathSerializer
from .qualibrate import QualibrateSettings, QualibrateSettingsSetup
from .qualibrate_app import QualibrateAppSettings, QualibrateAppSettingsSetup
from .remote_services import (
    JsonTimelineDBBase,
    QualibrateRunnerBase,
    RemoteServiceBase,
)
from .storage import StorageSettings, StorageSettingsSetup
from .storage_type import StorageType

__all__ = [
    "ActiveMachineSettings",
    "ActiveMachineSettingsSetup",
    "PathSerializer",
    "QualibrateSettings",
    "QualibrateSettingsSetup",
    "QualibrateAppSettings",
    "QualibrateAppSettingsSetup",
    "RemoteServiceBase",
    "QualibrateRunnerBase",
    "JsonTimelineDBBase",
    "StorageSettings",
    "StorageSettingsSetup",
    "StorageType",
]



from functools import cached_property
from typing import ClassVar

from pydantic import HttpUrl, field_serializer
from pydantic_core.core_schema import FieldSerializationInfo
from pydantic_settings import BaseSettings, SettingsConfigDict

__all__ = ["RemoteServiceBase", "JsonTimelineDBBase", "QualibrateRunnerBase"]


class RemoteServiceBase(BaseSettings):
    model_config: ClassVar[SettingsConfigDict] = SettingsConfigDict(
        extra="ignore",
    )
    address: HttpUrl
    timeout: float

    @field_serializer("address")
    def serialize_http_url(
        self, url: HttpUrl, _info: FieldSerializationInfo
    ) -> str:
        return str(url)

    @cached_property
    def address_with_root(self) -> str:
        address = str(self.address)
        return address if address.endswith("/") else address + "/"


class JsonTimelineDBBase(RemoteServiceBase):
    pass


class QualibrateRunnerBase(RemoteServiceBase):
    pass



from .file import get_config_file, read_config_file
from .models import (
    ActiveMachineSettings,
    ActiveMachineSettingsSetup,
    JsonTimelineDBBase,
    QualibrateAppSettings,
    QualibrateAppSettingsSetup,
    QualibrateRunnerBase,
    QualibrateSettings,
    QualibrateSettingsSetup,
    StorageSettings,
    StorageSettingsSetup,
    StorageType,
)
from .resolvers import get_config_path, get_settings
from .validation import get_config_model_or_print_error
from .vars import (
    ACTIVE_MACHINE_CONFIG_KEY,
    CONFIG_KEY,
    CONFIG_PATH_ENV_NAME,
    DEFAULT_ACTIVE_MACHINE_CONFIG_FILENAME,
    DEFAULT_CONFIG_FILENAME,
    DEFAULT_QUALIBRATE_APP_CONFIG_FILENAME,
    DEFAULT_QUALIBRATE_CONFIG_FILENAME,
    QUALIBRATE_CONFIG_KEY,
    QUALIBRATE_PATH,
)

__all__ = [
    "ActiveMachineSettings",
    "ActiveMachineSettingsSetup",
    "QualibrateSettings",
    "QualibrateSettingsSetup",
    "QualibrateAppSettings",
    "QualibrateAppSettingsSetup",
    "StorageSettings",
    "StorageSettingsSetup",
    "StorageType",
    "JsonTimelineDBBase",
    "QualibrateRunnerBase",
    "ACTIVE_MACHINE_CONFIG_KEY",
    "CONFIG_KEY",
    "QUALIBRATE_CONFIG_KEY",
    "CONFIG_PATH_ENV_NAME",
    "QUALIBRATE_PATH",
    "DEFAULT_ACTIVE_MACHINE_CONFIG_FILENAME",
    "DEFAULT_QUALIBRATE_APP_CONFIG_FILENAME",
    "DEFAULT_QUALIBRATE_CONFIG_FILENAME",
    "DEFAULT_CONFIG_FILENAME",
    "get_config_path",
    "get_config_file",
    "read_config_file",
    "get_settings",
    "get_config_model_or_print_error",
]



from pathlib import Path

CONFIG_KEY = "qualibrate_app"
QUALIBRATE_CONFIG_KEY = "qualibrate"
ACTIVE_MACHINE_CONFIG_KEY = "active_machine"
QUALIBRATE_PATH = Path().home() / ".qualibrate"
DEFAULT_CONFIG_FILENAME = "config.toml"
DEFAULT_ACTIVE_MACHINE_CONFIG_FILENAME = "active_machine.toml"
DEFAULT_QUALIBRATE_CONFIG_FILENAME = "qualibrate.toml"
DEFAULT_QUALIBRATE_APP_CONFIG_FILENAME = "qualibrate_app.toml"
CONFIG_PATH_ENV_NAME = "QUALIBRATE_APP_CONFIG_FILE"



import os
from functools import lru_cache
from pathlib import Path
from typing import Annotated, cast

from fastapi import Depends

from qualibrate_app.config.file import get_config_file
from qualibrate_app.config.models import (
    ActiveMachineSettings,
    QualibrateAppSettings,
    QualibrateSettings,
)
from qualibrate_app.config.validation import (
    get_config_model_or_print_error,
    get_config_solved_references_or_print_error,
)
from qualibrate_app.config.vars import (
    ACTIVE_MACHINE_CONFIG_KEY,
    CONFIG_KEY,
    CONFIG_PATH_ENV_NAME,
    QUALIBRATE_CONFIG_KEY,
)


@lru_cache
def get_config_path() -> Path:
    return get_config_file(os.environ.get(CONFIG_PATH_ENV_NAME))


@lru_cache
def get_settings(
    config_path: Annotated[Path, Depends(get_config_path)],
) -> QualibrateAppSettings:
    config = get_config_solved_references_or_print_error(config_path)
    if config is None:
        raise RuntimeError("Couldn't read config file")
    qs = get_config_model_or_print_error(
        config.get(QUALIBRATE_CONFIG_KEY, {}),
        QualibrateSettings,
        QUALIBRATE_CONFIG_KEY,
    )
    ams = get_config_model_or_print_error(
        config.get(ACTIVE_MACHINE_CONFIG_KEY, {}),
        ActiveMachineSettings,
        ACTIVE_MACHINE_CONFIG_KEY,
    )
    qas_config = config.get(CONFIG_KEY, {})
    qas_config.update(
        {QUALIBRATE_CONFIG_KEY: qs, ACTIVE_MACHINE_CONFIG_KEY: ams}
    )
    qas = get_config_model_or_print_error(
        qas_config, QualibrateAppSettings, CONFIG_KEY
    )
    if qas is None:
        raise RuntimeError("Couldn't read config file")
    return cast(QualibrateAppSettings, qas)



from typing import Any, Optional

from pydantic import Field

from qualibrate_app.api.core.models.base import ModelWithIdCreatedAt
from qualibrate_app.api.core.types import IdType


class SimplifiedSnapshot(ModelWithIdCreatedAt):
    parents: list[IdType]


class SimplifiedSnapshotWithMetadata(SimplifiedSnapshot):
    metadata: dict[str, Any]


class Snapshot(SimplifiedSnapshot):
    metadata: dict[str, Any] = Field(default_factory=dict)
    data: Optional[dict[str, Any]] = None



from os import sep as os_sep
from pathlib import Path

from pydantic_core.core_schema import FieldSerializationInfo

from qualibrate_app.config.references.resolvers import TEMPLATE_START

__all__ = ["PathSerializer"]


class PathSerializer:
    def serialize_path(self, path: Path, _info: FieldSerializationInfo) -> str:
        str_path = str(path)
        if os_sep == "/":
            return str_path
        t_start_idx = str_path.find(TEMPLATE_START)
        while t_start_idx != -1:
            t_end_idx = str_path.find("}", t_start_idx)
            if t_end_idx == -1:
                return str_path
            new_ref = str_path[t_start_idx:t_end_idx].replace(os_sep, "/")
            str_path = str_path.replace(
                str_path[t_start_idx:t_end_idx], new_ref
            )
            t_start_idx = str_path.find(TEMPLATE_START, t_end_idx + 1)
        return str_path



import os
import sys
from collections.abc import Mapping
from pathlib import Path
from typing import Any, Optional, cast

import click
import tomli_w
from click.core import ParameterSource

from qualibrate_app.config import (
    CONFIG_KEY,
    DEFAULT_CONFIG_FILENAME,
    QUALIBRATE_CONFIG_KEY,
    QUALIBRATE_PATH,
    ActiveMachineSettingsSetup,
    QualibrateSettingsSetup,
    StorageType,
    get_config_file,
)
from qualibrate_app.config.models import QualibrateAppSettingsSetup
from qualibrate_app.config.validation import (
    check_config_pre_v1_and_update,
    get_config_model_or_print_error,
)
from qualibrate_app.config.vars import ACTIVE_MACHINE_CONFIG_KEY

if sys.version_info[:2] < (3, 11):
    import tomli as tomllib
else:
    import tomllib


__all__ = ["config_command"]


def not_default(ctx: click.Context, arg_key: str) -> bool:
    return ctx.get_parameter_source(arg_key) in (
        ParameterSource.COMMANDLINE,
        ParameterSource.ENVIRONMENT,
    )


def get_config(config_path: Path) -> tuple[dict[str, Any], Path]:
    """Returns config and path to file"""
    config_file = get_config_file(config_path, raise_not_exists=False)
    if config_file.is_file():
        return tomllib.loads(config_file.read_text()), config_path
    return {}, config_file


def _config_from_sources(
    ctx: click.Context, from_file: dict[str, Any]
) -> dict[str, Any]:
    qualibrate_mapping = {
        k: k
        for k in (
            "static_site_files",
            # "user_storage",
            "metadata_out_path",
            # "storage_type",
            # "project",
        )
    }
    timeline_db_mapping = {
        "timeline_db_address": "address",
        "timeline_db_timeout": "timeout",
    }
    runner_mapping = {
        "runner_address": "address",
        "runner_timeout": "timeout",
    }
    for arg_key, arg_value in ctx.params.items():
        not_default_arg = not_default(ctx, arg_key)
        if arg_key in qualibrate_mapping:
            if not_default_arg or qualibrate_mapping[arg_key] not in from_file:
                from_file[qualibrate_mapping[arg_key]] = arg_value
        elif arg_key in timeline_db_mapping:
            if not_default_arg or (
                timeline_db_mapping[arg_key] not in from_file["timeline_db"]
            ):
                from_file["timeline_db"][timeline_db_mapping[arg_key]] = (
                    arg_value
                )
        elif arg_key in runner_mapping and (
            not_default_arg
            or (runner_mapping[arg_key] not in from_file["runner"])
        ):
            from_file["runner"][runner_mapping[arg_key]] = arg_value
    return from_file


def _print_config(data: Mapping[str, Any], depth: int = 0) -> None:
    try:
        max_key_len = max(map(len, map(str, data.keys())))
    except ValueError:
        max_key_len = 1
    click.echo(
        os.linesep.join(
            f"{' ' * 4 * depth}{f'{k} :':<{max_key_len + 3}} {v}"
            for k, v in data.items()
            if not isinstance(v, Mapping)
        )
    )
    mappings = filter(lambda x: isinstance(x[1], Mapping), data.items())
    for mapping_k, mapping_v in mappings:
        click.echo(f"{' ' * 4 * depth}{mapping_k} :")
        _print_config(mapping_v, depth + 1)


def _confirm(
    config_file: Path,
    qs_data: Mapping[str, Any],
    am_data: Mapping[str, Any],
    qas_data: Mapping[str, Any],
) -> None:
    click.echo(f"Config file path: {config_file}")
    click.echo(click.style("Generated config:", bold=True))
    _print_config(
        {
            "Qualibrate": qs_data,
            "Qualibrate app": qas_data,
            "Active machine": am_data,
        }
    )
    confirmed = click.confirm("Do you confirm config?", default=True)
    if not confirmed:
        click.echo(
            click.style(
                (
                    "The configuration has not been confirmed. "
                    "Rerun config script."
                ),
                fg="yellow",
            )
        )
        exit(1)


def write_config(
    config_file: Path,
    common_config: dict[str, Any],
    qss: QualibrateSettingsSetup,
    ams: ActiveMachineSettingsSetup,
    qass: QualibrateAppSettingsSetup,
    confirm: bool = True,
) -> None:
    qs_exported_data = qss.model_dump(exclude_none=True)
    am_exported_data = ams.model_dump(exclude_none=True)
    qas_exported_data = qass.model_dump()
    if confirm:
        _confirm(
            config_file, qs_exported_data, am_exported_data, qas_exported_data
        )
    qss.storage.location.mkdir(parents=True, exist_ok=True)
    if qss.project:
        project_path = qss.storage.location / qss.project
        project_path.mkdir(parents=True, exist_ok=True)
    if qss.log_folder:
        qss.log_folder.mkdir(parents=True, exist_ok=True)
    if not config_file.parent.exists():
        config_file.parent.mkdir(parents=True)
    common_config[CONFIG_KEY] = qas_exported_data
    common_config[ACTIVE_MACHINE_CONFIG_KEY] = am_exported_data
    common_config[QUALIBRATE_CONFIG_KEY] = qs_exported_data

    with config_file.open("wb") as f_out:
        tomli_w.dump(common_config, f_out)


@click.command(name="config")
@click.option(
    "--config-path",
    type=click.Path(
        exists=False,
        path_type=Path,
    ),
    default=QUALIBRATE_PATH / DEFAULT_CONFIG_FILENAME,
    show_default=True,
)
@click.option(
    "--auto-accept",
    type=bool,
    is_flag=True,
    default=False,
    show_default=True,
)
@click.option(
    "--overwrite",
    type=bool,
    default=False,
    is_flag=True,
    help="Ignore existing config and force overwrite values",
)
@click.option(
    "--static-site-files",
    type=click.Path(
        exists=True,
        file_okay=False,
        dir_okay=True,
        resolve_path=True,
        path_type=Path,
    ),
    default=Path(__file__).parents[2] / "qualibrate_static",
)
@click.option(
    "--storage-type",
    type=click.Choice([t.value for t in StorageType]),
    default=StorageType.local_storage.value,
    show_default=True,
)
@click.option(
    "--user-storage",
    type=click.Path(
        exists=False,
        resolve_path=True,
        path_type=Path,
    ),
    default=QUALIBRATE_PATH / "user_storage",
    help="Path to user storage directory with qualibrate data.",
    show_default=True,
)
@click.option(
    "--project",
    type=str,
    required=False,
)
@click.option(
    "--metadata-out-path",
    type=str,
    default="data_path",
    show_default=True,
)
@click.option(
    "--timeline-db-address",
    type=str,  # TODO: add type check for addr
    default="http://localhost:8000/",
    show_default=True,
)
@click.option(
    "--timeline-db-timeout",
    type=float,
    default=1.0,
    show_default=True,
)
@click.option(
    "--runner-address",
    type=str,  # TODO: add type check for addr
    default="http://localhost:8001/execution/",
    show_default=True,
)
@click.option(
    "--runner-timeout",
    type=float,
    default=1.0,
    show_default=True,
)
@click.pass_context
def config_command(
    ctx: click.Context,
    config_path: Path,
    auto_accept: bool,
    overwrite: bool,
    static_site_files: Path,
    storage_type: StorageType,
    log_path: Path,
    user_storage: Path,
    project: Optional[str],
    metadata_out_path: str,
    timeline_db_address: str,
    timeline_db_timeout: float,
    runner_address: str,
    runner_timeout: float,
) -> None:
    common_config, config_file = get_config(config_path)
    qualibrate_app_config = (
        common_config.get(CONFIG_KEY, {}) if not overwrite else {}
    )
    qapp_subconfigs = ("timeline_db", "runner")
    for subconfig in qapp_subconfigs:
        if subconfig not in qualibrate_app_config:
            qualibrate_app_config[subconfig] = {}

    qualibrate_app_config = _config_from_sources(ctx, qualibrate_app_config)
    (
        qualibrate_config,
        active_machine_config,
        qapp_config,
    ) = check_config_pre_v1_and_update(common_config, qualibrate_app_config)
    qss = get_config_model_or_print_error(
        qualibrate_config, QualibrateSettingsSetup, QUALIBRATE_CONFIG_KEY
    )
    ams = get_config_model_or_print_error(
        active_machine_config,
        ActiveMachineSettingsSetup,
        ACTIVE_MACHINE_CONFIG_KEY,
    )
    qass = get_config_model_or_print_error(
        qapp_config, QualibrateAppSettingsSetup, CONFIG_KEY
    )
    if qss is None or qass is None:
        return
    write_config(
        config_file,
        common_config,
        cast(QualibrateSettingsSetup, qss),
        cast(ActiveMachineSettingsSetup, ams),
        cast(QualibrateAppSettingsSetup, qass),
        confirm=not auto_accept,
    )
